/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             sharedcol
*  SCOPE :              Management of shared collection
*  FILE :               sharedcol.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

/*=
[DESCRIPTION :]
=*/

/* IMPORTS *******************************************************************/

#ifndef __sharedcol_HXX
#define __sharedcol_HXX

#ifdef __GNUG__
#pragma interface
#endif

#ifndef __obj_HXX
#include "obj.hxx"
#endif

#ifndef __list_HXX
#include "list.hxx"
#endif

#ifndef __listx_HXX
#include "listx.hxx"
#endif

#ifndef __dictx_HXX
#include "dictx.hxx"
#endif

#ifndef __iter_HXX
#include "iter.hxx"
#endif

class GenSharedCol ;
class GenSharedIterator ;

struct GenSharedMod {
  Ref<DictXGen>         _newElemList ;
  Ref<DictXGen>         _savElemList ;
  TableOfBits*          _delElemTab ;    
  Ref< ListX<ObjectX> > _delElemList ;
  void ((*_func)(GenSharedMod*)) ;
  
  GenSharedMod(const GenSharedCol&) ;
  void commit() ;
  ~GenSharedMod() ;
} ;


class GenSharedCol {
friend void GenSharedCol_commitAll() ;
friend void GenSharedCol_abortAll() ;
friend struct GenSharedMod ;
friend class GenSharedIterator ; 

  private :
  
  virtual DictXGen* savElemList() const {} ;
  virtual DictXGen* newElemList() const {} ;
  virtual void ((*registerCommit() const)(GenSharedMod*)) = 0 ; 
  virtual void init() const = 0 ;
  virtual int insert(const Ref<ObjectX>&) = 0 ;

  protected :

  TableOfBits* _delElemTab ;  
  static ListPtr<GenSharedMod> listmod ;
  static void commitAll() ;
  static void abortAll() ;
  int operator==(const GenSharedCol&) const ;
  int operator!=(const GenSharedCol& c) const {
    return ! (*this == c) ;
  }
  int insert(const LocalListX<ObjectX>&) ;
  int insert(const GenSharedCol&) ;
  virtual int member(const Ref<ObjectX>&) const = 0 ;

  public :

  ~GenSharedCol() ;
  int count() const ;
  void clear() ;  
} ;


class GenSharedIterator {

  protected :
  
  Iterator<ObjectX>*  _iterNew ;
  Iterator<ObjectX>*  _iterSav ;
  Iterator<ObjectX>*  _iterCurr ;
  TableOfBits*        _iterDel ;    
  
  public :
  
  GenSharedIterator(const GenSharedCol&) ;
  ~GenSharedIterator() ;

  Ref<ObjectX>& get() const ;
  void next() ;
  void prev() ;
  void reset() ;
  int  more() const ;
} ;


#include "sharedcol_tmpl.hxx"
#include "sharedcol_tmpl.cc"

#define yoo_declare_SharedSet(ELEM) \
     template class Iterator<ELEM> ; \
     template class LocalListX<ELEM> ; \
     template class ListX<ELEM> ; \
     template class DictX<ELEM, OID, (long)&ELEM::_oid> ; \
     template class SharedCol<ELEM, OID, (long)&ELEM::_oid> ; \
     template class SharedIterator<ELEM, OID, (long)&ELEM::_oid> ; \
     template class SharedSet<ELEM> ; \
     template class SharedSetIterator<ELEM> ; \
     template class Ref<DictX<ELEM, OID, yoo_index(ELEM::_oid)> > ;


#define yoo_declare_SharedDict(ELEM, KEYT, KEY) \
     template class Iterator<ELEM> ; \
     template class LocalListX<ELEM> ; \
     template class ListX<ELEM> ; \
     template class DictX<ELEM, KEYT, (long)&ELEM::KEY> ; \
     template class SharedCol<ELEM, KEYT, (long)&ELEM::KEY> ; \
     template class SharedIterator<ELEM, KEYT, (long)&ELEM::KEY> ; \
     template class SharedDict<ELEM, KEYT, (long)&ELEM::KEY> ; \
     template class SharedDictIterator<ELEM, KEYT, (long)&ELEM::KEY> ; 

#endif

